// Fragment Shader  file "phongShader.frag"

#version 330 core
//Blinn-Phong Shading with textures
//with three light sources (1. point light, 2-3. spot light)

uniform mat3 reflectionCoefficient;  //1.column ambient 2.column diffuse, 3.column specular
uniform float shininess;

uniform sampler2D colorMap;  //contains texture
uniform mat3 spotDirection; //in which direction the spot light shines, in world coordinates

in vec3 normal;			  //normal vector of fragment
in vec3 cameraDirection;  //direction fragment to camera, in world coordinates
in vec2 texCoords;

in mat3 lightIntensity;   //intensity of light, includes color of light
in mat3 lightDirection;   //direction fragment to light, in world coordinates

uniform float spotCosCutoff[3]; //how big is the light cone
uniform float spotExponent[3]; //how strong light decreases from center to walls of light cone

out vec4 out_color; //final color of fragment

uniform sampler2DShadow shadowMap1;
in vec4 SM_tex_coords1;	//shadow map coords
uniform sampler2DShadow shadowMap2;
in vec4 SM_tex_coords2;	//shadow map coords
uniform sampler2DShadow shadowMap3;
in vec4 SM_tex_coords3;	//shadow map coords



void main(void) {

	vec3 diffuse = vec3(0,0,0);
	vec3 specular = vec3(0,0,0);
	vec3 texColor = texture(colorMap,texCoords.st).rgb;
	vec3 ambient = vec3(0.3,0.3,0.3)* reflectionCoefficient[0] * texColor;

float shadow[3];

/*
vec3 direction[8];

direction[0] = -lightDirection[0]+vec3(0.03,0.03,-0.03);
direction[1] = -lightDirection[0]+vec3(-0.03,0.03,0.03);
direction[2] = -lightDirection[0]+vec3(-0.03,-0.03,0.03);
direction[3] = -lightDirection[0]+vec3(0.03,-0.03,-0.03);
direction[4] = -lightDirection[0]+vec3(0.03,-0.03,0.03);
direction[5] = -lightDirection[0]+vec3(-0.03,0.03,-0.03);
direction[6] = -lightDirection[0]+vec3(0.03,0.03,0.03);
direction[7] = -lightDirection[0]+vec3(-0.03,-0.03,-0.03);


shadow[0] = 0;
float curr_fragment_dist_to_light = length(lightDirection[0]);

for(int samples = 0; samples<1; samples++) {

	float smallest_dist_to_light = texture(shadowMap1, direction[samples]).r;
	smallest_dist_to_light = smallest_dist_to_light * 120;

	if(smallest_dist_to_light > curr_fragment_dist_to_light || curr_fragment_dist_to_light>120)
	shadow[0] += 1;

}
shadow[0] = shadow[0]/1;
*/
	
	shadow[0]  = textureProj(shadowMap1, SM_tex_coords1+vec4(0.01,0.01,0,0));
	shadow[0]  += textureProj(shadowMap1, SM_tex_coords1+vec4(-0.01,0.01,0,0));
	shadow[0]  += textureProj(shadowMap1, SM_tex_coords1+vec4(-0.01,-0.01,0,0));
	shadow[0]  += textureProj(shadowMap1, SM_tex_coords1+vec4(0.01,-0.01,0,0));
	shadow[0] = (shadow[0])/4;
	

	//no filtering for spot lights so far
	shadow[1]= textureProj(shadowMap2, SM_tex_coords2);
	shadow[2]= textureProj(shadowMap3, SM_tex_coords3);

	
	for(int i=0;i<3;i++) {

	if (shadow[i]==0)
	continue;
	

	//diffuse part
		float N_L = dot(normalize(normal),normalize(lightDirection[i]));
		if (N_L > 0) {

				float spotEffect = 1;

		//		if(i>0) {
				spotEffect = dot(normalize(spotDirection[i]), normalize(-lightDirection[i]));
					if (spotEffect < spotCosCutoff[i])
					continue;
					
					spotEffect = pow(spotEffect, spotExponent[i]);
		//		}

				diffuse += shadow[i]*reflectionCoefficient[1] * ((texColor*spotEffect*lightIntensity[i]) * N_L);

				//specular part
				vec3 halfway = normalize(cameraDirection)+normalize(lightDirection[i]);
				float N_H = dot(normalize(normal),normalize(halfway));
				if (N_H > 0) {
					float highlight = pow(N_H,shininess);
					specular += shadow[i]*reflectionCoefficient[2] * (spotEffect*lightIntensity[i]* highlight);
				}
			
		}
	}

	//adding all parts together to get the result
	out_color = vec4(ambient + diffuse + specular,1.0);


}

