// Vertex Shader  file "phongShader.vert"

#version 330 core

//Blinn-Phong Shading with textures
//with three light sources (1. point light, 2-3. spot light)

//geometry uniforms
uniform mat3 normalMatrix;
uniform mat4 modelMatrix;
uniform mat4 mvpMatrix;

//camera uniforms
uniform vec4 cameraPos;		//in world coordinates

//light uniforms
uniform mat3 lightPosition;  //in world coordinates
uniform mat3 lightColor;
uniform mat3 attenuation;

//per vertex attributes
in  vec4 in_position;		//in local coordinates
in  vec3 in_normal;
in  vec2 in_texCoords;

//variables for fragment shader		
out mat3 lightIntensity;	//intensity of light, includes color
out mat3 lightDirection;	//direction vertex to light
out vec3 cameraDirection;  //direction vertex to camera
out vec3 normal;		   //transformed normal vector of vertex
out vec2 texCoords;

uniform mat4 texMat_model1;
uniform mat4 texMat_model2;
uniform mat4 texMat_model3;
out vec4 SM_tex_coords1;
out vec4 SM_tex_coords2;
out vec4 SM_tex_coords3;

void main(void) {


	SM_tex_coords1 = texMat_model1*in_position;
	SM_tex_coords2 = texMat_model2*in_position;
	SM_tex_coords3 = texMat_model3*in_position;

	texCoords = in_texCoords;
	normal =   normalMatrix * in_normal;

	vec4 worldPos = modelMatrix * in_position;

	cameraDirection = vec3(cameraPos-worldPos);
	

	for(int i=0;i<3;i++) {

		lightDirection[i] = lightPosition[i] - worldPos.xyz;
		float distanceToLight = length(lightDirection[i]);
		
		lightIntensity[i] = lightColor[i]*(1/(attenuation[i].x + attenuation[i].y*distanceToLight + attenuation[i].z*distanceToLight*distanceToLight));
	}


	gl_Position = mvpMatrix * in_position;

}
