// Fragment Shader  file "phongShader.frag"

#version 130
//Blinn-Phong Shading with textures

uniform mat3 reflectionCoefficient;  //1.column ambient 2.column diffuse, 3.column specular
uniform float shininess;

uniform sampler2D colorMap;  //contains texture
uniform mat3 spotDirection; //in which direction the spot light shines, in world coordinates

in vec3 normal;			  //normal vector of fragment
in vec3 cameraDirection;  //direction fragment to camera, in world coordinates
in vec2 texCoords;

in mat3 lightIntensity;   //intensity of light, includes color of light
in mat3 lightDirection;   //direction fragment to light, in world coordinates

//uniform float spotCosCutoff[3];

//uniform float spotExponent[3]; //how decrease from center to walls


out vec4 out_color; //final color of fragment

void main(void) {

float spotExponent[3];
float spotCosCutoff[3];
spotExponent[0] = 1;
spotExponent[1] = 20;
spotExponent[2] = 5;
spotCosCutoff[0] = -1;
spotCosCutoff[1] = 0.3;
spotCosCutoff[2] = 0.2;


	vec3 diffuse = vec3(0,0,0);
	vec3 specular = vec3(0,0,0);
	vec3 texColor = texture(colorMap,texCoords.st).rgb;

	//ambient part
	vec3 ambient = vec3(0.3,0.3,0.3)* reflectionCoefficient[0] * texColor;

	for(int i=0;i<3;i++) {

	//diffuse part
		float N_L = dot(normalize(normal),normalize(lightDirection[i]));
		if (N_L > 0) {

			float spotEffect = dot(normalize(spotDirection[i]), normalize(-lightDirection[i]));
			
			if (spotEffect > spotCosCutoff[i]) {

				spotEffect = pow(spotEffect, spotExponent[i]);
				diffuse += reflectionCoefficient[1] * ((texColor*spotEffect*lightIntensity[i]) * N_L);

				//specular part
				vec3 halfway = normalize(cameraDirection)+normalize(lightDirection[i]);
				float N_H = dot(normalize(normal),normalize(halfway));
				if (N_H > 0) {
					float highlight = pow(N_H,shininess);
					specular += reflectionCoefficient[2] * (spotEffect*lightIntensity[i]* highlight);
				}
			}
		}
	}


	//adding all parts together to get the result
	out_color = vec4(ambient + diffuse + specular,1.0);

}

