// Vertex Shader  file "phongShader.vert"

#version 130

//Blinn-Phong Shading with textures

//geometry uniforms
uniform mat3 normalMatrix;
uniform mat4 modelMatrix;

//camera uniforms
uniform vec4 cameraPos;		//in world coordinates
uniform mat4 viewProjectionMatrix;

//light uniforms
uniform mat3 lightPosition;  //in world coordinates
uniform mat3 lightColor;
uniform mat3 attenuation;

//per vertex attributes
in  vec4 in_position;		//in local coordinates
in  vec3 in_normal;
in  vec2 in_texCoords;

//variables for fragment shader		
/*smooth*/ out mat3 lightIntensity;	//intensity of light, includes color
/*smooth*/ out mat3 lightDirection;	//direction vertex to light
/*smooth*/ out vec3 cameraDirection;  //direction vertex to camera
/*smooth*/ out vec3 normal;		   //transformed normal vector of vertex
/*smooth*/ out vec2 texCoords;


void main(void) {

	texCoords = in_texCoords;
	normal =   normalMatrix * in_normal;

	vec4 worldPos = modelMatrix * in_position;

	cameraDirection = vec3(cameraPos-worldPos);

	for(int i=0;i<3;i++) {

		lightDirection[i] = lightPosition[i] - worldPos.xyz;
		float distanceToLight = length(lightDirection[i]);
		lightIntensity[i] = lightColor[i]*(1/(attenuation[i].x + attenuation[i].y*distanceToLight + attenuation[i].z*distanceToLight*distanceToLight));
	}

	gl_Position = viewProjectionMatrix * worldPos;

}
